import itertools
import time
from datetime import datetime

from rich.table import Table
from rich.text import Text
from rich.live import Live
from rich import box, print as rich_print

from irishrail import IrishRail


COLOR_GREEN = "light_green"
COLOR_ORANGE = "sandy_brown"
COLOR_GRAY = "gray70"


def format_duein(station):
    due = station.get("Duein", "")
    if not due:
        return ""

    return f"{due} min"


def split_per_direction(data):
    northbound = southbound = []
    for train in data:
        if train["Direction"] == "Southbound":
            southbound.append(train)
        else:
            northbound.append(train)

    return northbound, southbound


def render_title(station_name):
    NEWLINE = "\n"
    DIVIDER = " - "

    return Text.assemble(
        NEWLINE,
        (station_name.title(), "bold"),
        NEWLINE,
        ("Northbound", "light_green bold"),
        DIVIDER,
        ("Southbound", "sandy_brown bold"),
    )


def render_station(station, northbound, southbound, updated_at):
    title = render_title(station)

    table = Table(
        title=title,
        caption=f"Updated at: {updated_at}",
        box=box.DOUBLE_EDGE,
        title_style="white bold",
        caption_style=COLOR_GRAY,
        border_style=COLOR_GRAY,
        row_styles=["", "dim"],
    )

    table.add_column("Destination", style=COLOR_GREEN)
    table.add_column("Due", style=COLOR_GREEN)
    table.add_column("Destination", style=COLOR_ORANGE)
    table.add_column("Due", style=COLOR_ORANGE)

    for north, south in itertools.zip_longest(northbound, southbound, fillvalue={}):
        table.add_row(
            north.get("Destination", ""),
            format_duein(north),
            south.get("Destination", ""),
            format_duein(south),
        )

    return table


def render_station_not_found(station):
    client = IrishRail()
    stations = client.list_stations()
    stations = [station["StationDesc"].lower() for station in stations]

    if station.lower() not in stations:
        message = Text("Station not found. Use command `stations` to list all stations")
    else:
        message = Text.assemble(
            "No trains going to ",
            (station, "bold"),
            " station",
        )

    rich_print(message)


def render_live(station):
    client = IrishRail()
    data = client.next_trains(station)
    if not data:
        render_station_not_found(station)
        return None
    updated_at = datetime.now()

    northbound, southbound = split_per_direction(data)
    return render_station(station, northbound, southbound, updated_at)


def live(station, follow):
    REFRESH_PER_SECOND = 30

    with Live(render_live(station)) as live:
        while follow:
            time.sleep(REFRESH_PER_SECOND)
            table = render_live(station)
            if not table:
                break
            live.update(table)



import click
import time
from rich.live import Live
from rich import print

from irishrail import commands


@click.group()
def cli():
    ...


@cli.command()
def stations():
    stations = commands.stations()
    print(stations)


@cli.command()
@click.option("-s", "--station")
@click.option("-f", "--follow", is_flag=True, default=False)
def live(station, follow):
    commands.live(station, follow)


if __name__ == "__main__":
    cli()



import httpx
import xmltodict


class IrishRail:
    def _request(self, url):
        response = httpx.get(url)
        data = xmltodict.parse(response.content)
        return data

    def next_trains(self, station):
        data = self._request(
            f"http://api.irishrail.ie/realtime/realtime.asmx/getStationDataByNameXML?StationDesc={station}"
        )
        try:
            data = data["ArrayOfObjStationData"]["objStationData"]
        except KeyError:
            # IrishRail API returns 202 for stations that doesn't exists.
            return []

        return sorted(data, key=lambda s: s["Duein"])

    def list_stations(self):
        data = self._request(
            "http://api.irishrail.ie/realtime/realtime.asmx/getAllStationsXML"
        )
        data = data["ArrayOfObjStation"]["objStation"]
        return sorted(data, key=lambda s: s["StationDesc"])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['irishrail', 'irishrail.commands']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'httpx>=0.23.0,<0.24.0',
 'rich>=12.4.4,<13.0.0',
 'xmltodict>=0.13.0,<0.14.0']

entry_points = \
{'console_scripts': ['irishrail = irishrail.cli:cli']}

setup_kwargs = {
    'name': 'irishrail',
    'version': '0.0.3a0',
    'description': '',
    'long_description': None,
    'author': 'Marco Rougeth',
    'author_email': 'rougeth@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from rich.columns import Columns

from irishrail import IrishRail


def _print_station(station):
    name = station["StationDesc"]
    code = station["StationCode"]
    return f"{name} ({code})"


def stations():
    client = IrishRail()
    data = client.list_stations()

    stations = [_print_station(station) for station in data]
    columns = Columns(stations, equal=True, expand=True)
    return columns
